import random

import sc2
from sc2 import Difficulty, Race
from sc2.ids.ability_id import EFFECT_CHRONOBOOSTENERGYCOST, SMART
from sc2.ids.unit_typeid import ADEPTPHASESHIFT, BROODLING, DISRUPTORPHASED, EGG, LARVA, NEXUS, PROBE
from sc2.player import Bot, Computer


class meepmeep(sc2.BotAI):
    def __init__(self):
        self.home_mineral = None
        self.enemy_mineral = None
        self.mining_probe_tag = 0
        self.ignore_type = {ADEPTPHASESHIFT, DISRUPTORPHASED, EGG, LARVA, BROODLING}
        self.surrendered: bool = False

    async def on_step(self, iteration):
        if self.surrendered:
            return
        self.enemy_mineral = self.mineral_field.closest_to(self.enemy_start_locations[0])
        self.home_mineral = self.mineral_field.closest_to(self.start_location)
        if iteration == 0:
            self._client.game_step = 1
            home_probe = self.workers.furthest_to(self.main_base_ramp.depot_in_middle)
            self.mining_probe_tag = home_probe.tag
            self.mineralwalk_home(home_probe)
        attackers = self.workers.filter(lambda unit: unit.tag != self.mining_probe_tag)
        if not attackers:
            self.surrendered = True
        self.probe_micro(attackers)
        self.nexus_control()

    def probe_micro(self, probes):
        enemies = self.enemy_units.filter(
            lambda unit: not unit.is_structure
            and not unit.is_flying
            and unit.type_id not in self.ignore_type
            and (
                unit.distance_to(self.enemy_start_locations[0]) < 30
                or any(unit.distance_to(other_unit) < 10 for other_unit in self.enemy_units)
            )
        )
        visible_structures = self.enemy_structures.filter(
            lambda unit: unit.is_visible and not unit.is_flying
        )
        attack = all(probe.shield > 0 for probe in probes) or any(unit.ground_range > 1 for unit in enemies)
        current_targets = set()
        for probe in probes:
            if not enemies:
                if not visible_structures:
                    self.mineralwalk_enemy(probe)
                else:
                    target = self.attack_low_hp(probe, visible_structures)
                    self.do(probe.attack(target))
                continue
            if probe.shield <= 11 and enemies.closer_than(3, probe):
            # if not attack and enemies.closer_than(3, probe):
                self.mineralwalk_home(probe)
                continue

            if probe.weapon_cooldown <= self._client.game_step:
                possible_targets = enemies.in_attack_range_of(probe)
                if possible_targets:
                    target_fire_enemies = [enemy for enemy in possible_targets if enemy.tag in current_targets]
                    if target_fire_enemies:
                        possible_targets = target_fire_enemies
                    target = self.attack_low_hp(probe, possible_targets)
                    current_targets.add(target.tag)
                    self.do(probe.attack(target))
                    continue
                # else:
                #     target = enemies.closest_to(probe)
                #     dist = probe.distance_to(target)
                #     if dist > 3 and target.distance_to(self.enemy_mineral) < probe.distance_to(self.enemy_mineral):
                #         self.mineralwalk_enemy(probe)
                #         continue
            target = enemies.closest_to(probe)
            self.do(probe.attack(target))

    def mineralwalk_home(self, probe):
        self.do(probe(SMART, self.home_mineral))

    def mineralwalk_enemy(self, probe):
        self.do(probe(SMART, self.enemy_mineral))

    def nexus_control(self):
        nexus = self.townhalls.first
        if self.minerals >= 50 and self.supply_left >= 1:
            self.do(nexus.build(PROBE))
        if not nexus.is_idle and nexus.energy >= 50:
            self.do(nexus(EFFECT_CHRONOBOOSTENERGYCOST, nexus))

    def attack_low_hp(self, unit, enemies):
        current_min = 10000
        current_dist = 10000
        target = None
        for enemy in enemies:
            hp = enemy.health + enemy.shield
            if hp > current_min:
                continue
            elif hp == current_min:
                dist = unit.distance_to(enemy)
                if dist >= current_dist:
                    continue
                else:
                    current_min = hp
                    current_dist = dist
                    target = enemy
            else:
                current_min = hp
                current_dist = unit.distance_to(enemy)
                target = enemy
        return target


def main():
    race = random.choice([sc2.Race.Zerg, sc2.Race.Terran, sc2.Race.Protoss, sc2.Race.Random])
    random_map = random.choice(
        [
            # "AutomatonLE",
            "Acropolis",
            # "BlueshiftLE",
            # "CeruleanFallLE",
            # "KairosJunctionLE",
            # "ParaSiteLE",
            # "PortAleksanderLE",
            # "StasisLE",
            # "DarknessSanctuaryLE", # 4 player map, bot is not ready for that
        ]
    )
    sc2.run_game(
        sc2.maps.get(random_map), [Bot(Race.Protoss, meepmeep()), Computer(race, Difficulty.VeryHard)], realtime=False
    )


if __name__ == "__main__":
    main()
